import tensorflow as tf
import numpy as np
import ast
import json
import os
import argparse
from PIL import Image

class CocoToTfrecords():
    def __init__(self, in_coco_path, out_tfrecord_dir):
        self.in_coco_path = in_coco_path
        self.out_tfrecord_dir = out_tfrecord_dir
        self.annotations = None
        self.image_ppts = None
        self.image_paths = []
        self.categories = None
        self.images = None

        self.coco_to_tfrecord()

    #methods to convert values to type compatible with tf.train.Example
    def _bytes_feature(self, value):
        """Returns a bytes_list from a string / byte."""
        if isinstance(value, type(tf.constant(0))):
            value = value.numpy() # BytesList won't unpack a string from an EagerTensor.
        return tf.train.Feature(bytes_list=tf.train.BytesList(value=[value]))

    def _float_feature(self, value):
        """Returns a float_list from a float / double."""
        return tf.train.Feature(float_list=tf.train.FloatList(value=[value]))

    def _float_feature_list(self, value):
        """Returns a float_list from a float / double."""
        return tf.train.Feature(float_list=tf.train.FloatList(value=value))

    def _int64_feature(self, value):
        """Returns an int64_list from a bool / enum / int / uint."""
        return tf.train.Feature(int64_list=tf.train.Int64List(value=[value]))

    def _image_feature(self, value):
        """Returns a bytes_list from a string / byte."""
        return tf.train.Feature(bytes_list=tf.train.BytesList(value=[tf.io.encode_jpeg(value).numpy()]))

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def create_example(self, image, path, annotation):
        feature = {
            "image": self._image_feature(image),
            "path": self._bytes_feature(tf.io.serialize_tensor(path)),
            "area": self._float_feature(annotation["area"]),
            "bbox": self._float_feature_list(annotation["bbox"]),
            "category_id": self._int64_feature(annotation["category_id"]),
            "id": self._int64_feature(annotation["id"]),
            "image_id": self._int64_feature(annotation["image_id"]),
        }
        return tf.train.Example(features=tf.train.Features(feature=feature))

    def get_coco_info(self, ann_path):
        with open(ann_path, 'r') as f:
            info = json.load(f)
            
        self.annotations = info['annotations']
        self.image_ppts = info['images']
        self.categories = info['categories']

        for id in range(len(self.image_ppts)):
            self.image_paths.append(self.image_ppts[id]['file_name'])
    
    def coco_to_tfrecord(self):
        print('Starting conversion...')
        self.out_tfrecord_dir = self.valid_path(self.out_tfrecord_dir)
        out_tfrecord_path = os.path.join(self.out_tfrecord_dir, 'annotations.tfrecord')
        #get root directory of annotation file
        coco_root_dir = os.path.dirname(self.in_coco_path)
        #get info from coco file
        self.get_coco_info(self.in_coco_path)
        for annotation in self.annotations:
            with tf.io.TFRecordWriter(out_tfrecord_path) as writer:
                image_id = annotation['image_id']
                image_path = os.path.join(coco_root_dir, self.image_paths[image_id])
                # image_path = f'{coco_root_dir}\{self.image_paths[image_id]}'
                image = tf.io.decode_jpeg(tf.io.read_file(image_path))
                example = self.create_example(image, image_path, annotation)
                writer.write(example.SerializeToString())  
        
        print('Writing to label map file')
        label_map_path = os.path.join(self.out_tfrecord_dir, 'label_map.txt')
        self.generate_labelmap(label_map_path)
        print('All done!')

    
    def generate_labelmap(self, label_map_path):
        with open(label_map_path, 'w') as f:
            self.categories = [str(cat) for cat in self.categories]
            f.writelines(self.categories)
            



from .cocoToTfrecords import CocoToTfrecords
from .labelmetococo import LabelmeToCoco
from .labelmeToDarknetTxt import LabelmeToDarknetTxt
from .labelmeToKerasRetinanetCsv import LabelmeToRetinanetCsv
from .labelmeToMask import LabelmeToMask
from .labelmeToTfCsv import LabelmeToTfCsv
from .labelmeToXml import LabelmeToXml
from .labelmeToYoloTxt import LabelmeToYoloTxt
from .labelmeToYoloV3KerasTxt import LabelmeToYolov3KerasTxt
from .xmlToDarknetTxt import XmlToDarknetTxt
from .xmlToJson import XmlToJson
from .xmlToKerasRetinanetCsv import XmlToKerasRetinanetCsv
from .xmlToTfCsv import XmlToTfCsv
from .xmlToYoloTxt import XmlToYoloTxt
from .xmlToYoloV3KerasTxt import XmlToYolov3KerasTxt
from .xmlToCoco import XmlToCoco
from .rectvisConverter import RectvisionConverter



import os
import argparse
import shutil
import xml.etree.ElementTree as ET
import json
from datetime import datetime
import numpy as np
import glob
from PIL import Image, ImageDraw


class NpEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.integer):
            return float(obj)
        if isinstance(obj, np.floating):
            return float(obj)
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)

class XmlToCoco(object):
    def __init__(self, ann_dir, out_coco_dir, project_desc=""):
        self.ann_dir = ann_dir
        self.xml_ann = glob.glob(os.path.join(self.ann_dir, "*.xml"))
        self.out_coco_dir = out_coco_dir
        self.save_json_path = os.path.join(self.valid_path(self.out_coco_dir), 'annotations.json')
        self.project_desc = project_desc
        self.info = {}
        self.images = []
        self.categories = []
        self.annotations = []
        self.label = []
        self.annID = 1
        self.height = 0
        self.width = 0

        self.save_json()
    
    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def gen_info(self):
        self.info["description"] = self.project_desc
        date = datetime.now()
        self.info["version"] = "1.0"
        # self.info["year"] = date.year
        self.info["date_created"] = str(date)

    def image(self, data, num):
        '''This method gets the width, height, id and filename for an image and 
        stores it in the image dictionary'''
        image = {}
        #data is ET.parse(ann_path).getroot()
        #get image width and height
        size = data.find('size')
        height = int(size.findtext('width'))
        width = int(size.findtext('height'))
        size = None
        image["height"] = height
        image["width"] = width
        image["id"] = num
        #get image file name
        image["file_name"] = data.findtext('filename').split("/")[-1]

        self.height = height
        self.width = width

        return image

    def data_transfer(self):
        for num, xml_file in enumerate(self.xml_ann):
            #open xml ann file and get fileroot
            data = ET.parse(xml_file).getroot()
            self.images.append(self.image(data, num))
            for obj in data.findall('object'):
                label = obj.findtext('name')
                if label not in self.label:
                    self.label.append(label)
                bndbox = obj.find('bndbox')
                xmin = int(float(bndbox.findtext('xmin')))
                ymin = int(float(bndbox.findtext('ymin'))) 
                xmax = int(float(bndbox.findtext('xmax')))
                ymax = int(float(bndbox.findtext('ymax')))
                
                points = [[xmin, ymin], [xmax, ymax]]
                # print(points)
                self.annotations.append(self.annotation(points, label, num))
                self.annID += 1

        # Sort all text labels so they are in the same order across data splits.
        self.label.sort()
        for label in self.label:
            self.categories.append(self.category(label))
        for annotation in self.annotations:
            annotation["category_id"] = self.getcatid(annotation["category_id"])

    def category(self, label):
        category = {}
        category["supercategory"] = label
        category["id"] = len(self.categories)
        category["name"] = label
        return category

    def getcatid(self, label):
        for category in self.categories:
            if label == category["name"]:
                return category["id"]
        print("label: {} not in categories: {}.".format(label, self.categories))
        exit()
        return -1

    def annotation(self, points, label, num):
        annotation = {}
        contour = np.array(points)
        #x and y coordinates
        x = contour[:, 0]
        y = contour[:, 1]
        #area of bounding polgon
        # area = 0.5 * np.abs(np.dot(x, np.roll(y, 1)) - np.dot(y, np.roll(x, 1)))
        # annotation["segmentation"] = [list(np.asarray(points).flatten())]
        annotation["iscrowd"] = 0
        # annotation["area"] = area
        annotation["image_id"] = num
        [[xmin, ymin], [xmax, ymax]] = points
        o_width = xmax - xmin
        o_height = ymax - ymin
        annotation["bbox"] = [xmin, ymin, o_width, o_height]

        annotation["category_id"] = label  # self.getcatid(label)
        annotation["id"] = self.annID
        return annotation

    def data2coco(self):
        data_coco = {}
        data_coco["info"] = self.info
        data_coco["images"] = self.images        
        data_coco["annotations"] = self.annotations
        data_coco["categories"] = self.categories
        return data_coco

    def copy_images(self):
        for path in os.listdir(self.ann_dir):
            if not path.endswith(".xml"):
                # Copy image to out_coco_dir
                shutil.copy(os.path.join(self.ann_dir, path), self.out_coco_dir)

    def save_json(self):
        print("save coco json")
        self.gen_info()
        self.data_transfer()
        self.data_coco = self.data2coco()

        print(self.save_json_path)
        os.makedirs(
            os.path.dirname(os.path.abspath(self.save_json_path)), exist_ok=True
        )
        json.dump(self.data_coco, open(self.save_json_path, "w"), indent=4, cls=NpEncoder)
        self.copy_images()



#this script gets project's annotation and call conversion script to convert to specified format.
from json import loads
import os, uuid, shutil
import getpass
import jwt, cv2, random
import requests
from tqdm import tqdm
from .rectvisConverterHelper import GenerateAnnotation

def download_url_to_disk(image_url):
    try:
        os.makedir('tmp')
    except Exception as err:
        pass
    file_disk_path = './tmp/' + str(uuid.uuid4()) + ".jpg"
    r = requests.get(image_url, stream = True)

    # Check if the csv was retrieved successfully
    if r.status_code == 200:
        # Set decode_content value to True, otherwise the downloaded image file's size will be zero.
        r.raw.decode_content = True

        with open(os.path.join(file_disk_path),'wb') as f:
            shutil.copyfileobj(r.raw, f)
    
    else:
        pass 
    
    return file_disk_path

def get_image_widith_height(image_url):
    file_path = download_url_to_disk(image_url)
    img = cv2.imread(file_path)
    height, width, channel = img.shape
    os.remove(file_path)
    return height, width, channel

class RectvisionConverter():
    def __init__(self, train_split=0.6, test_split=0.2, validation_split=0.2, export_format='yolo-txt', token = None): 
        if token == None:       
            self.token = getpass.getpass('Enter Token: ')
        else:
            self.token = token
        self.endpoint = "https://test.backend.app.rectvision.com/api/v1/"
        base_url = self.endpoint + 'projects/decode-token?token='
        request_url = base_url + self.token
        self.login_response = requests.get(request_url)
        while not self.login_response.ok:
            print("Something went wrong! Ensure the right token was entered!")
            self.token = getpass.getpass('Enter Token: ')
            request_url = base_url + self.token
            self.login_response = requests.get(request_url)
        self.export_format = export_format
        self.train_split = train_split
        self.test_split = test_split
        self.validation_split = validation_split
        self.user_id, self.project_id, self.user_token = self.get_creds()
        self.rectvision_converter()
    
    def get_creds(self):
        creds = loads(self.login_response.text)['data']['decoded']
        user_id, project_id, user_token = creds['user_id'], creds['project_id'], creds['token']
        return user_id, project_id, user_token

    def get_db_info(self):
        #get project details from get-a-project endpoint
        base_url = self.endpoint + 'projects/'
        request_url = base_url + self.project_id
        headers={'Authorization':self.user_token}
        response = requests.get(request_url, headers=headers)
        current_project = loads(response.text)['data']['project']

        # annotations = current_project['annotations']
        labels = [label['value'] for label in current_project['labels']]
        if current_project['annotation_choice'] == 'Object_detection':
            shape_type = 'rectangle'

        return labels, shape_type 

    def get_annotations(self):
        #get annotations from endpoint
        base_url = self.endpoint + 'projects/'
        request_url_files = base_url + self.project_id + '/files?limit=1000000000'
        request_url_annotations = base_url + self.project_id + '/annotations?limit=1000000000'
        headers={'Authorization':self.user_token}
        file_response = requests.get(request_url_files, headers=headers)
        annotation_response = requests.get(request_url_annotations, headers=headers)
        file_properties = loads(file_response.text)
        annotation_properties = loads(annotation_response.text)
        file_prop = file_properties['data']['files']
        annotation_prop = annotation_properties['data']['annotations']

        rearranged_annotations = {}
        total_data = len(annotation_prop)
        data_split = random.choices(['train', 'test', 'val'], weights = [self.train_split, self.test_split, self.validation_split], k=total_data)
        for data_tag, annon_prop in tqdm(zip(data_split, annotation_prop), desc = 'getting annotations'):
            image_name = annon_prop['file']['name']
            if image_name not in rearranged_annotations:
                rearranged_annotations[image_name] = {}
            else:
                continue
            if 'points' not in rearranged_annotations[image_name]:
                rearranged_annotations[image_name]['points'] = []
            if 'labels' not in rearranged_annotations[image_name]:
                rearranged_annotations[image_name]['labels'] = []
            if 'data_tag' not in rearranged_annotations[image_name]:
                rearranged_annotations[image_name]['data_tag'] = None

            rearranged_annotations[image_name]['image_id'] = annon_prop['file_id']
            rearranged_annotations[image_name]['image_url'] = annon_prop['file']['url']
            rearranged_annotations[image_name]['image_height'] =annon_prop['image_height']
            rearranged_annotations[image_name]['image_width'] = annon_prop['image_width']
            rearranged_annotations[image_name]['image_channels'] = 3
            rearranged_annotations[image_name]['points'].append(annon_prop['points'])
            rearranged_annotations[image_name]['labels'].append(annon_prop['label']['value'])
            rearranged_annotations[image_name]['data_tag'] = data_tag
        return rearranged_annotations
      



    def rectvision_converter(self):
        labels, shape_type = self.get_db_info()
        rearranged_annotations = self.get_annotations()        
        #convert data
        GenerateAnnotation(export_format=self.export_format, annotations=rearranged_annotations, labels=labels, shape_type=shape_type,
                        database = 'User Provided', train_ratio=self.train_split, test_ratio=self.test_split, valid_ratio=self.validation_split)

        print('Annotations saved to results directory in the current working directory')




import getpass
import requests
import jwt
import os
from dotenv import load_dotenv
from json import loads

class GetCredentials():
    def __init__(self, token):
        load_dotenv()
        self.token = token
        self.user_id, self.project_id, self.user_token = self.get_creds()

    def get_creds(self):
        key = os.getenv('JWT_KEY')
        creds = jwt.decode(self.token, key, algorithm = "HS256")['payload']
        user_id, project_id, user_token = creds['user_id'], creds['project_id'], creds['token']
        return user_id, project_id, user_token
   
    # def get_project_credential(self):
    #     base_url = 'https://backend.app.rectvision.com/api/v1/projects/'
    #     request_url = base_url + self.project_id
    #     headers={'Authorization':self.user_token}
    #     response = requests.get(request_url, headers=headers)
    #     if response.ok:
    #         print("Project {} credentials: \n".format(project_id))            
    #         response_details = loads(response.text)['data']['project']
    #         #extract project details from response_details
    #         return response_details           
    #     else:
    #         print("Something went wrong! Make sure project_id and token entered are correct!")
    #         print(response.text)


import argparse
import json
from re import A
import xml.etree.ElementTree as ET
import os
import glob
import shutil
import base64
import numpy as np
import labelme

class NpEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.integer):
            return int(obj)
        if isinstance(obj, np.floating):
            return float(obj)
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)

class XmlToJson():   
    def __init__(self, ann_dir, out_json_dir):
        self.ann_dir = ann_dir
        self.out_json_dir = out_json_dir
        self.shape_type = 'rectangle'
        self.ppts = []
        self.current_img_path = None

        self.xml_to_json()

    def extract_info_from_xml(self, ann_path):
        self.ppts = []
        annotation_root = ET.parse(ann_path).getroot()
        img_path = annotation_root.findtext('path')
        if img_path is None:
            img_path = annotation_root.findtext('filename')
        self.current_img_path = img_path
        for obj in annotation_root.findall('object'):
            #get label
            label = obj.findtext('name')
            #get coordinates
            bndbox = obj.find('bndbox')
            xmin = int(float(bndbox.findtext('xmin')))
            ymin = int(float(bndbox.findtext('ymin'))) 
            xmax = int(float(bndbox.findtext('xmax')))
            ymax = int(float(bndbox.findtext('ymax')))
            points = [[xmin, ymin], [xmax, ymax]]
            self.ppts.append([self.current_img_path, label, self.shape_type, points])
       

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def xml_to_json(self):
        print('Starting conversion...')
        self.out_json_dir = self.valid_path(self.out_json_dir)
        for ann_path in glob.glob(os.path.join(self.ann_dir, '*.xml')):
            
            output_json_dict = {
                'shapes': [],
                'imagePath': ''
            }
            #get image info
            self.extract_info_from_xml(ann_path)
            #copy image from ann_dir and save to out_json_dir
            shutil.copy(os.path.join(self.ann_dir, self.current_img_path), self.out_json_dir)
            for ppt in self.ppts:
                img_path, label, shape_type, points = ppt
                #update imagePath
                output_json_dict['imagePath']=img_path
                data = labelme.LabelFile.load_image_file(os.path.join(self.ann_dir, img_path))
                output_json_dict['imageData']=base64.b64encode(data).decode('utf-8')
                #shapes info
                shapes_info = {
                    'label':'',
                    'points': [],
                    'shape_type': ''
                }
                #update shapes_info
                shapes_info['label'] = label
                shapes_info['points'] = points
                shapes_info['shape_type'] = shape_type
                output_json_dict['shapes'].append(shapes_info)
            #write to json file
            out_json_path = os.path.splitext(os.path.join(self.out_json_dir, self.current_img_path))[0] + '.json'
            with open(out_json_path, 'w') as f:
                output_json = json.dumps(output_json_dict, indent=4, cls=NpEncoder)
                f.write(output_json)
        print('All done!')
        



import numpy as np
import json
import glob
import os
import argparse
import ast
import shutil


class LabelmeToYolov3KerasTxt():   
    def __init__(self, label_to_id_file_path, ann_dir, out_txt_dir):
        self.label_to_id = self.read_dictionary(label_to_id_file_path)
        self.ann_dir = ann_dir
        self.out_txt_dir = out_txt_dir
        self.ppts = []
        self.current_img_path = None

        self.json_to_txt()

    def read_dictionary(self, dict_file_path):
        with open(dict_file_path, "r") as data:
            dictionary = ast.literal_eval(data.read())  
        return dictionary     
    
    def extract_info_from_json(self, ann_path):
        self.ppts = []
        with open(ann_path, 'r') as fp:
            data = json.load(fp)
            #get path to original image
            # img_path = os.path.join(self.ann_dir, data['imagePath'])
            self.current_img_path = data['imagePath']
            #get annotation coordinates and labels
            for shapes in data['shapes']:
                points = shapes['points']
                #get label for each set of points
                label = shapes['label']
                label_id = self.label_to_id[label]
                x_min, y_min, x_max, y_max = self.pointsTobbox(points)
                self.ppts.append([x_min, y_min, x_max, y_max, label_id])
        

    def pointsTobbox(self, points):
        x_min,y_min = points[0][:]
        x_max,y_max = points[1][:]        
        return x_min, y_min, x_max, y_max

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path


    def json_to_txt(self):
        print('Starting conversion...')
        self.out_txt_dir = self.valid_path(self.out_txt_dir)
        out_txt_path = os.path.join(self.out_txt_dir, 'annotations.txt')
        with open(out_txt_path, 'w') as f:
            for ann_path in glob.glob(os.path.join(self.ann_dir, '*.json')):
                self.extract_info_from_json(ann_path)
                f.writelines([self.current_img_path, ' ', ' '.join(str(','.join(str(ppt) for ppt in ppts))
                                                                        for ppts in self.ppts), '\n']) 

                #copy image from ann_dir and save to out_txt_dir
                shutil.copy(os.path.join(self.ann_dir, self.current_img_path), self.out_txt_dir)      
                
        print('All done!')
        


    

    


import xml.etree.ElementTree as ET
import json
import glob
import shutil
import requests
import os
import argparse
import ast
from labelme import utils


class LabelmeToXml():   
    def __init__(self, label_to_id_file_path, ann_dir, out_xml_dir, database='User Provided'):
        self.label_to_id = self.read_dictionary(label_to_id_file_path)
        self.ann_dir = ann_dir
        self.out_xml_dir = out_xml_dir
        self.database = database
        self.ppts = []
        self.current_img_path = None
        self.current_img_width = 0
        self.current_img_height = 0
        self.current_img_depth = 0

        self.json_to_xml()

    def get_xml_template(self):
        request_url = 'https://rectvision.s3.amazonaws.com/xmlTemplate.xml'
        response = requests.get(request_url)
        open('template.xml', 'wb').write(response.content)

    def read_dictionary(self, dict_file_path):
        with open(dict_file_path, "r") as data:
            dictionary = ast.literal_eval(data.read())  
        return dictionary     
    
    def extract_info_from_json(self, ann_path):
        self.ppts = []
        with open(ann_path, 'r') as fp:
            data = json.load(fp)
            #get path to original image
            self.current_img_path = data['imagePath']
            #get  image dimension from imageData
            self.current_img_height, self.current_img_width, self.current_img_depth = utils.img_b64_to_arr(data["imageData"]).shape
            #get annotation coordinates and labels
            for shapes in data['shapes']:
                points = shapes['points']
                #get label for each set of points
                label = shapes['label']
                #convert label to ID from label_to_id dictionary
                label_id = self.label_to_id[label]
                #get properties of the bounding box
                x_min, x_max, y_min, y_max = self.pointsTobbox(points)
                self.ppts.append([label_id,x_min, x_max, y_min, y_max])

    def pointsTobbox(self, points):
        '''This method converts the points array to the properties of the object's bounding box'''
        x_min,y_min = points[0][:]
        x_max,y_max = points[1][:]        
        return x_min, x_max, y_min, y_max

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def json_to_xml(self):
        self.get_xml_template()
        print('Starting conversion...')
        tree = ET.parse('template.xml')
        root = tree.getroot()
        #checkk validity of output text dir and create one if it doesn't exist
        self.out_xml_dir = self.valid_path(self.out_xml_dir)
        #iterate over all json files in self.ann_dir and extract required info from each
        for ann_path in glob.glob(os.path.join(self.ann_dir, '*.json')):
            self.extract_info_from_json(ann_path)

            #copy image from ann_dir and save to out_xml_dir
            shutil.copy(os.path.join(self.ann_dir, self.current_img_path), self.out_xml_dir)

            #get output xml file
            out_xml_path = os.path.splitext(os.path.join(self.out_xml_dir, self.current_img_path))[0] + '.xml'
            #modify xml template
            folder = root.find('folder')
            folder.text = self.ann_dir

            fname = root.find('filename')
            fname.text = self.current_img_path

            src = root.find('source')
            database = src.find('database')
            database.text = self.database

            size = root.find('size')
            width = size.find('width')
            width.text = str(self.current_img_width)
            height = size.find('height')
            height.text = str(self.current_img_height)
            depth = size.find('depth')
            depth.text = str(self.current_img_depth)
            
            #write objects as specific annotations
            for ppt in self.ppts:
                #append new object
                obj = ET.SubElement(root, 'object')

                name = ET.SubElement(obj, 'name')
                name.text = str(ppt[0])

                pose = ET.SubElement(obj, 'pose')
                pose.text = 'Unspecified'

                bndbox = ET.SubElement(obj, 'bndbox')

                xmin = ET.SubElement(bndbox, 'xmin')
                xmin.text = str(int(ppt[1]))
                xmax = ET.SubElement(bndbox, 'xmax')
                xmax.text = str(int(ppt[2]))
                ymin = ET.SubElement(bndbox, 'ymin')
                ymin.text = str(int(ppt[3]))
                ymax = ET.SubElement(bndbox, 'ymax')
                ymax.text = str(int(ppt[4]))
            
            #save annotation to out_xml_path
            tree.write(out_xml_path)
                
        print('All done!')
        


    

    


import numpy as np
import json
import glob
import os
import argparse
import ast
import csv
import shutil
from labelme import utils


class LabelmeToTfCsv():   
    def __init__(self, ann_dir, out_csv_dir):
        self.ann_dir = ann_dir
        self.out_csv_dir = out_csv_dir
        self.ppts = []
        self.current_img_path = None
        self.current_img_width = 0
        self.current_img_height = 0

        self.json_to_csv()
    
    def extract_info_from_json(self, ann_path):
        self.ppts = []
        with open(ann_path, 'r') as fp:
            data = json.load(fp)
            #get path to original image
            self.current_img_path = data['imagePath']
            #get  image dimension from imageData
            self.current_img_height, self.current_img_width = utils.img_b64_to_arr(data["imageData"]).shape[:2]
            #get annotation coordinates and labels
            for shapes in data['shapes']:
                points = shapes['points']
                #get label for each set of points
                label = shapes['label']
                x_min, y_min, x_max, y_max = self.pointsTobbox(points)
                self.ppts.append([self.current_img_path, label, self.current_img_width, self.current_img_height, x_min, y_min, x_max, y_max])
        

    def pointsTobbox(self, points):
        x_min,y_min = points[0][:]
        x_max,y_max = points[1][:]        
        return x_min, y_min, x_max, y_max

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path


    def json_to_csv(self):
        print('Starting conversion...')
        self.out_csv_dir = self.valid_path(self.out_csv_dir)
        out_csv_path = os.path.join(self.out_csv_dir, 'annotations.csv')
        with open(out_csv_path, 'w') as f:
            writer = csv.writer(f)

            for ann_path in glob.glob(os.path.join(self.ann_dir, '*.json')):
                self.extract_info_from_json(ann_path)
                writer.writerows(self.ppts)    
                #copy image from ann_dir and save to out_csv_dir
                shutil.copy(os.path.join(self.ann_dir, self.current_img_path), self.out_csv_dir)   
                
        print('All done!')
        



    

    


import numpy as np
import json
import glob
import os
import argparse
import cv2
import ast
import PIL.Image

class LabelmeToMask():   
    def __init__(self, label_to_colour_file_path, ann_dir, out_mask_dir):
        self.label_to_colour = self.read_dictionary(label_to_colour_file_path)
        self.ann_dir = ann_dir
        self.out_mask_dir = out_mask_dir
        self.all_points = []
        self.current_labels = []
        self.current_img_path = None

        self.json_to_mask()

    def read_dictionary(self, dict_file_path):
        with open(dict_file_path, "r") as data:
            dictionary = ast.literal_eval(data.read())  
        return dictionary     
    
    def extract_info_from_json(self, ann_path):
        self.all_points = []
        self.current_labels = []
        with open(ann_path, 'r') as fp:
            data = json.load(fp)
            #get path to original image
            self.current_img_path = data['imagePath']
            #get annotation coordinates and labels
            for shapes in data['shapes']:
                #get boundind box coordinates
                points = shapes['points']
                self.all_points.append(np.array(points, np.int32))
                #get label for each set of points
                label = shapes['label']
                self.current_labels.append(label)

    def draw_mask(self, image, points, colour):
        #reshape array points
        points = points.reshape((-1, 1, 2))
        #draw polygon around point and colour it colour
        image = cv2.fillPoly(image, [points], color=colour)
        return image

    def mask_background(self, image, points, bkg_colour):
        mask_value = 255
        stencil = np.zeros(image.shape[:2], dtype=np.uint8)
        cv2.fillPoly(stencil, points, mask_value)
        sel = stencil != mask_value
        image[sel] = bkg_colour
        return image
    
    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def json_to_mask(self):
        print('Starting conversion...')
        #check validity of out_mask_dir and create it if it doesn't exist
        self.out_mask_dir = self.valid_path(self.out_mask_dir)
        for ann_path in glob.glob(os.path.join(self.ann_dir, '*.json')):
            self.extract_info_from_json(ann_path)
            image = cv2.imread(os.path.join(self.ann_dir, self.current_img_path))

            for idx, points in enumerate(self.all_points):
                label = self.current_labels[idx]
                colour = self.label_to_colour[label]
                image = self.draw_mask(image, points, colour)
            #mask out background with the colour black
            image = self.mask_background(image, self.all_points, (0,0,0))
            out_path = os.path.join(self.out_mask_dir, self.current_img_path)
            cv2.imwrite(out_path, image)
        print('All done!')




    

    


from setuptools import setup, find_packages
import codecs
import os, sys, subprocess


here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '3.0.59'
DESCRIPTION = 'A low-code tool to help create your own AI'
LONG_DESCRIPTION = 'A package that allows to build your own computer visions and NLP systems'


requirement_path = here + '/requirements.txt'
install_requires = [] 
if os.path.isfile(requirement_path):
    with open(requirement_path) as f:
        install_requires = f.read().splitlines()

# Setting up
setup(
    name="rectvision",
    version=VERSION,
    author="Rectlabs Inc",
    author_email="rectanglenet@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=install_requires,
    keywords=['python', 'NLP', 'computer vision', 'deep learning', 'AI'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


import numpy as np
from zipfile import ZipFile
from pathlib import Path
import json
import random
import base64
import glob
import requests
import labelme
import shutil
import os
import csv
import ast
import xml.etree.ElementTree as ET
from tqdm import tqdm


class NpEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.integer):
            return int(obj)
        if isinstance(obj, np.floating):
            return float(obj)
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)

class GenerateAnnotation():   
    def __init__(self, export_format, annotations, labels=[], shape_type='rectangle', 
                database = 'User Provided', train_ratio=0.7, test_ratio=0.2, valid_ratio=0.1):
        self.endpoint = "https://test.backend.app.rectvision.com/api/v1/"
        self.annotations = annotations
        self.export_format = export_format
        self.shape_type = shape_type
        self.database = database
        self.train_ratio = train_ratio
        self.test_ratio = test_ratio
        self.valid_ratio = valid_ratio
        self.ppts = []
        self.labels = labels
        #create results directory
        self.valid_path('dataset')
        #generate labelmap
        # TODO
        self.label_to_id_file_path = os.path.join('dataset', 'labelmap.txt')
        self.train_path = './dataset/train'
        self.test_path = './dataset/test'
        self.validation_path = './dataset/val' #self.valid_path(os.path.join('dataset', 'validation'))
        self.images_folder = self.valid_path(os.path.join('dataset', 'images'))
        self.labels_folder = self.valid_path(os.path.join('dataset', 'labels'))
        self.generate_labelmap(self.labels, self.label_to_id_file_path)
        self.label_to_id = self.read_dictionary(self.label_to_id_file_path)


        #insert which annotation conversion method should be called
        if self.export_format == 'labelme-json':
            self.rectjson_to_json()
        elif self.export_format == 'darknet-txt':
            self.rectjson_to_darknetTxt()
        elif self.export_format == 'yolo-txt':
            self.rectjson_to_yoloTxt()
        elif self.export_format == 'yolo-seg-txt':
            self.rectjson_to_yoloSegTxt()
        elif self.export_format == 'xml':
            self.rectjson_to_xml()
        elif self.export_format == 'yolov3-keras-txt':
            self.rectjson_to_yolov3kerasTxt()
        elif self.export_format == 'keras-retinanet-csv':
            self.rectjson_to_kerasRetinanetCsv()
        elif self.export_format == 'tf-csv':
            self.rectjson_to_tfCsv()

        
    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def replace_extension(self, path, new_extension):
        p = Path(path)
        extensions = ''.join(p.suffixes)
        return str(p).replace(extensions, new_extension)

    def download_image(self, image_url, save_dir):
        filename = image_url.split("/")[-1]
        r = requests.get(image_url, stream = True)

        # Check if the image was retrieved successfully
        if r.status_code == 200:
            # Set decode_content value to True, otherwise the downloaded image file's size will be zero.
            r.raw.decode_content = True
            
            # Open a local file with wb ( write binary ) permission.
            out_dir = self.valid_path(save_dir)
            with open(os.path.join(out_dir, filename),'wb') as f:
                shutil.copyfileobj(r.raw, f)
      
        else:
            print('Image Couldn\'t be retreived')

    def compress_annotations(self, zip_path):
        #get files in folder 
        ann_directories = [self.train_path, self.test_path, self.validation_path]
        self.ann_file_paths = [self.label_to_id_file_path]
        for directory in ann_directories:
            for root, directories, files in os.walk(directory):
                for filename in files:
                    filepath = os.path.join(root, filename)
                    self.ann_file_paths.append(filepath)

        #write files in file_paths to a zipfile
        with ZipFile(zip_path, 'w') as zip:
            for file in self.ann_file_paths:
                zip.write(file)
        return zip_path

    def upload_annotation(self):
        #this method zips, uploads zipped annotation file to s3 and updates db url
        #zip projects directory
        zip_path = self.compress_annotations(zip_path=os.path.join('results', 'annotations.zip'))
        #post annotation to endpoint   
        base_url = self.endpoint + 'projects/'
        request_url = base_url + self.project_id + '/zip-file'
        headers={'Authorization':self.user_token}
        fo = open(zip_path, 'rb')
        files = {'file': ('zip', fo, 'application/zip')}
        response = requests.post(request_url, files=files, headers=headers)
        if response.ok:
            url = json.loads(response.text)['data']['upload']['Location']
            # print("Upload completed successfully!")
            # print(response.text)
            fo.close()
            shutil.rmtree('results', ignore_errors=False, onerror=None)
            return url
        else:
            print("Something went wrong!")
            print(response.text)       
        

    
    def generate_labelmap(self, labels, label_to_id_file_path='label2id.txt'):
        label_ids = list(range(0, len(labels)))
        label_to_id = dict(zip(labels, label_ids))

        with open(label_to_id_file_path, 'w') as data:
            data.write(str(label_to_id))
    
    def read_dictionary(self, dict_file_path):
        with open(dict_file_path, "r") as data:
            dictionary = ast.literal_eval(data.read())  
        return dictionary 

    def extract_info_from_json(self):
        self.ppts = []
        for image_name, image_data in tqdm(self.annotations.items(), desc = 'converting data'):
            current_img_path = image_name
            current_img_width, current_img_height, current_img_depth = image_data['image_width'], image_data['image_height'], image_data['image_channels']
            points = image_data['points']
            labels = image_data['labels']
            image_ppts = []

            if self.export_format == 'labelme-json':
                for idx, point in enumerate(points):
                    label = labels[idx]
                    point_x = [coord[0] for coord in point]
                    point_y = [coord[1] for coord in point]
                    x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                    image_ppts.append([current_img_path, label, self.shape_type, [[x_min, y_min], [x_max, y_max]]])
                
            
            elif self.export_format == 'darknet-txt':
                for idx, point in enumerate(points):
                    label = labels[idx]
                    label_id = self.label_to_id[label]
                    point_x = [coord[0] for coord in point]
                    point_y = [coord[1] for coord in point]
                    x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                    width, height = x_max - x_min, y_max - y_min
                    x_center, y_center,  = x_min + (width/2), y_min + (height/2)
                    image_ppts.append([current_img_path, label_id, 
                                        np.abs(x_center)/current_img_width, 
                                        np.abs(y_center)/current_img_height, 
                                        np.abs(width)/current_img_width, 
                                        np.abs(height)/current_img_height, '\n' ])
                

            elif self.export_format == 'yolo-txt':
                for idx, point in enumerate(points):
                    label = labels[idx]
                    label_id = self.label_to_id[label]
                    point_x = [coord[0] for coord in point]
                    point_y = [coord[1] for coord in point]
                    x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                    width, height = x_max - x_min, y_max - y_min
                    x_center, y_center,  = (x_min + x_max)/2, (y_min + y_max)/2
                    image_ppts.append([current_img_path, label_id, x_center/current_img_width, 
                                       y_center/current_img_height, width/current_img_width, height/current_img_height, '\n' ])

            elif self.export_format == "yolo-seg-txt":
                 for idx, point in enumerate(points):
                    label = labels[idx]
                    label_id = self.label_to_id[label]
                    point_x = [coord[0] for coord in point]
                    point_y = [coord[1] for coord in point]
                    x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                    width, height = x_max - x_min, y_max - y_min
                    # x_center, y_center,  = (x_min + x_max)/2, (y_min + y_max)/2
                    pk = []
                    for x, y in zip(point_x, point_y):
                        pk.append(np.abs(x)/current_img_width)
                        pk.append(np.abs(y)/current_img_height)
                    pre_data = [current_img_path, label_id]
                    pre_data.extend(pk)
                    pre_data.append('\n')
                    image_ppts.append(pre_data)

            elif self.export_format == 'yolov3-keras-txt':
                for idx, point in enumerate(points):
                    label = labels[idx]
                    label_id = self.label_to_id[label]
                    point_x = [coord[0] for coord in point]
                    point_y = [coord[1] for coord in point]
                    x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                    image_ppts.append([current_img_path, x_min, y_min, x_max, y_max, label_id])
            
            elif self.export_format == 'xml':
                for idx, point in enumerate(points):
                    label = labels[idx]
                    point_x = [coord[0] for coord in point]
                    point_y = [coord[1] for coord in point]
                    x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                    image_ppts.append([current_img_path, current_img_width, 
                                        current_img_height, current_img_depth,
                                        label, self.shape_type, x_min, x_max, y_min, y_max])

            elif self.export_format == 'keras-retinanet-csv':
                for idx, point in enumerate(points):
                    label = labels[idx]
                    point_x = [coord[0] for coord in point]
                    point_y = [coord[1] for coord in point]
                    x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                    image_ppts.append([current_img_path, x_min, y_min, x_max, y_max, label])

            elif self.export_format == 'tf-csv':
                for idx, point in enumerate(points):
                    label = labels[idx]
                    label_id = self.label_to_id[label]
                    point_x = [coord[0] for coord in point]
                    point_y = [coord[1] for coord in point]
                    x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                    image_ppts.append([current_img_path, label,
                                        current_img_width, current_img_height,
                                        x_min, y_min, x_max, y_max])
            self.ppts.append(image_ppts)

       
    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def split(self, arr, split1, split2, split3):
        random.shuffle(arr)
        arr_length = len(arr)
        arr1 = arr[:round(split1*arr_length)]
        arr2 = arr[round(split1*arr_length):round((split1+split2)*arr_length)]
        arr3 = arr[round((split1+split2)*arr_length):]
        return arr1, arr2, arr3

    def rectjson_to_json(self):
        print('Starting conversion...')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        for image_ppt in self.ppts_train:
            #download image
            image_name = image_ppt[0][0]
            image_url = self.annotations[image_name]['image_url']
            self.download_image(image_url, self.train_path)
            output_json_dict = {
                'shapes': [],
                'imagePath': ''
            }
            for ppt in image_ppt:
                img_path, label, shape_type, points = ppt
                #update imagePath
                output_json_dict['imagePath']=img_path
                data = labelme.LabelFile.load_image_file(os.path.join(self.train_path, img_path))
                output_json_dict['imageData']=base64.b64encode(data).decode('utf-8')
                #shapes info
                shapes_info = {
                    'label':'',
                    'points': [],
                    'shape_type': ''
                }
                #update shapes_info
                shapes_info['label'] = label
                shapes_info['points'] = points
                shapes_info['shape_type'] = shape_type
                output_json_dict['shapes'].append(shapes_info)
            #write to json file
            train_out_annotation_dir = self.valid_path(self.train_path)
            out_annotation_path = os.path.join(train_out_annotation_dir, self.replace_extension(image_ppt[0][0], '.json'))
            with open(out_annotation_path, 'w') as f:
                output_json = json.dumps(output_json_dict, indent=4, cls=NpEncoder)
                f.write(output_json)
        for image_ppt in self.ppts_test:
            #download image
            image_name = image_ppt[0][0]
            image_url = self.annotations[image_name]['image_url']
            self.download_image(image_url, self.test_path)
            output_json_dict = {
                'shapes': [],
                'imagePath': ''
            }
            for ppt in image_ppt:
                img_path, label, shape_type, points = ppt
                #update imagePath
                output_json_dict['imagePath']=img_path
                data = labelme.LabelFile.load_image_file(os.path.join(self.test_path, img_path))
                output_json_dict['imageData']=base64.b64encode(data).decode('utf-8')
                #shapes info
                shapes_info = {
                    'label':'',
                    'points': [],
                    'shape_type': ''
                }
                #update shapes_info
                shapes_info['label'] = label
                shapes_info['points'] = points
                shapes_info['shape_type'] = shape_type
                output_json_dict['shapes'].append(shapes_info)
            #write to json file
            test_out_annotation_dir = self.valid_path(self.test_path)
            out_annotation_path = os.path.join(test_out_annotation_dir, self.replace_extension(image_ppt[0][0], '.json'))
            with open(out_annotation_path, 'w') as f:
                output_json = json.dumps(output_json_dict, indent=4, cls=NpEncoder)
                f.write(output_json)
        for image_ppt in self.ppts_valid:
            #download image
            image_name = image_ppt[0][0]
            image_url = self.annotations[image_name]['image_url']
            self.download_image(image_url, self.validation_path)
            output_json_dict = {
                'shapes': [],
                'imagePath': ''
            }
            for ppt in image_ppt:
                img_path, label, shape_type, points = ppt
                #update imagePath
                output_json_dict['imagePath']=img_path
                data = labelme.LabelFile.load_image_file(os.path.join(self.validation_path, img_path))
                output_json_dict['imageData']=base64.b64encode(data).decode('utf-8')
                #shapes info
                shapes_info = {
                    'label':'',
                    'points': [],
                    'shape_type': ''
                }
                #update shapes_info
                shapes_info['label'] = label
                shapes_info['points'] = points
                shapes_info['shape_type'] = shape_type
                output_json_dict['shapes'].append(shapes_info)
            #write to json file
            validation_out_annotation_dir = self.valid_path(self.validation_path)
            out_annotation_path = os.path.join(validation_out_annotation_dir, self.replace_extension(image_ppt[0][0], '.json'))
            with open(out_annotation_path, 'w') as f:
                output_json = json.dumps(output_json_dict, indent=4, cls=NpEncoder)
                f.write(output_json)
        print('All done!')

    def rectjson_to_darknetTxt(self):
        print('Starting conversion...')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        for image_ppt in self.ppts_train:   
            #download image
            image_name = image_ppt[0][0]
            image_url = self.annotations[image_name]['image_url']
            self.download_image(image_url, os.path.join(self.train_path, 'images'))         
            #write to txt file
            train_out_annotation_dir = self.valid_path(os.path.join(self.train_path, 'annotations'))
            out_annotation_path = os.path.join(train_out_annotation_dir, self.replace_extension(image_ppt[0][0], '.txt'))
            with open(out_annotation_path, 'w') as f:
                for ppts in image_ppt:
                    f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        for image_ppt in self.ppts_test: 
            #download image
            image_name = image_ppt[0][0]
            image_url = self.annotations[image_name]['image_url']
            self.download_image(image_url, os.path.join(self.test_path, 'images'))           
            #write to txt file
            test_out_annotation_dir = self.valid_path(os.path.join(self.test_path, 'annotations'))
            out_annotation_path = os.path.join(test_out_annotation_dir, self.replace_extension(image_ppt[0][0], '.txt'))
            with open(out_annotation_path, 'w') as f:
                for ppts in image_ppt:
                    f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        for image_ppt in self.ppts_valid:        
            #download image
            image_name = image_ppt[0][0]
            image_url = self.annotations[image_name]['image_url']
            self.download_image(image_url, os.path.join(self.validation_path, 'images'))    
            #write to txt file
            validation_out_annotation_dir = self.valid_path(os.path.join(self.validation_path, 'annotations'))
            out_annotation_path = os.path.join(validation_out_annotation_dir, self.replace_extension(image_ppt[0][0], '.txt'))
            with open(out_annotation_path, 'w') as f:
                for ppts in image_ppt:
                    f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        print('All done!') 
    
    def rectjson_to_yoloSegTxt(self):
        self.ppts = []
        for image_name, image_data in tqdm(self.annotations.items(), desc = 'converting data'):
            current_img_path = image_name
            current_img_width, current_img_height, current_img_depth = image_data['image_width'], image_data['image_height'], image_data['image_channels']
            points = image_data['points']
            labels = image_data['labels']
            image_ppts = []

            for idx, point in enumerate(points):
                label = labels[idx]
                label_id = self.label_to_id[label]
                point_x = [coord[0] for coord in point]
                point_y = [coord[1] for coord in point]
                x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                width, height = x_max - x_min, y_max - y_min
                pk = []
                for x, y in zip(point_x, point_y):
                    pk.append(np.abs(x)/current_img_width)
                    pk.append(np.abs(y)/current_img_height)
                pre_data = [current_img_path, label_id]
                pre_data.extend(pk)
                pre_data.append('\n')
                image_ppts.append(pre_data)

            # #split to train, test, validation
            decision = self.annotations[image_name]['data_tag']
            
            if decision == 'train':
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.train_path, 'images'))       
                #write to txt file
                train_out_annotation_dir = self.valid_path(os.path.join(self.train_path, 'labels'))
                out_annotation_path = os.path.join(train_out_annotation_dir, self.replace_extension(image_ppts[0][0], '.txt'))
                with open(out_annotation_path, 'w') as f:
                    for ppts in image_ppts:
                        f.write(' '.join(str(ppt) for ppt in ppts[1:]))

            elif decision == 'test':
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.test_path, 'images'))        
                #write to txt file
                test_out_annotation_dir = self.valid_path(os.path.join(self.test_path, 'labels'))
                out_annotation_path = os.path.join(test_out_annotation_dir, self.replace_extension(image_ppts[0][0], '.txt'))
                with open(out_annotation_path, 'w') as f:
                    for ppts in image_ppts:
                        f.write(' '.join(str(ppt) for ppt in ppts[1:]))

            else:
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.validation_path, 'images'))    
                #write to txt file
                validation_out_annotation_dir = self.valid_path(os.path.join(self.labels_folder, 'labels'))
                out_annotation_path = os.path.join(validation_out_annotation_dir, self.replace_extension(image_ppts[0][0], '.txt'))
                with open(out_annotation_path, 'w') as f:
                    for ppts in image_ppts:
                        f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        print('All done!')  


    def rectjson_to_yoloTxt(self):
        self.ppts = []
        for image_name, image_data in tqdm(self.annotations.items(), desc = 'converting data'):
            current_img_path = image_name
            current_img_width, current_img_height, current_img_depth = image_data['image_width'], image_data['image_height'], image_data['image_channels']
            points = image_data['points']
            labels = image_data['labels']
            image_ppts = []

            for idx, point in enumerate(points):
                label = labels[idx]
                label_id = self.label_to_id[label]
                point_x = [coord[0] for coord in point]
                point_y = [coord[1] for coord in point]
                x_min, y_min, x_max, y_max = min(point_x), min(point_y), max(point_x), max(point_y)
                width, height = x_max - x_min, y_max - y_min
                x_center, y_center,  = x_min + (width/2), y_min + (height/2)
                image_ppts.append([current_img_path, label_id, 
                                    np.abs(x_center)/current_img_width, 
                                    np.abs(y_center)/current_img_height, 
                                    np.abs(width)/current_img_width, 
                                    np.abs(height)/current_img_height, '\n' ])

            # #split to train, test, validation
            decision = self.annotations[image_name]['data_tag']
            
            if decision == 'train':
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.images_folder, 'train'))       
                #write to txt file
                train_out_annotation_dir = self.valid_path(os.path.join(self.labels_folder, 'train'))
                out_annotation_path = os.path.join(train_out_annotation_dir, self.replace_extension(image_ppts[0][0], '.txt'))
                with open(out_annotation_path, 'w') as f:
                    for ppts in image_ppts:
                        f.write(' '.join(str(ppt) for ppt in ppts[1:]))

            elif decision == 'test':
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.images_folder, 'test'))        
                #write to txt file
                test_out_annotation_dir = self.valid_path(os.path.join(self.labels_folder, 'test'))
                out_annotation_path = os.path.join(test_out_annotation_dir, self.replace_extension(image_ppts[0][0], '.txt'))
                with open(out_annotation_path, 'w') as f:
                    for ppts in image_ppts:
                        f.write(' '.join(str(ppt) for ppt in ppts[1:]))

            else:
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.images_folder, 'val'))    
                #write to txt file
                validation_out_annotation_dir = self.valid_path(os.path.join(self.labels_folder, 'val'))
                out_annotation_path = os.path.join(validation_out_annotation_dir, self.replace_extension(image_ppts[0][0], '.txt'))
                with open(out_annotation_path, 'w') as f:
                    for ppts in image_ppts:
                        f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        print('All done!')  

    def rectjson_to_yolov3kerasTxt(self):
        print('Starting conversion...')
        #write to txt file
        train_out_annotation_dir = self.valid_path(os.path.join(self.train_path, 'annotations'))
        out_annotation_path = os.path.join(train_out_annotation_dir, 'annotations.txt')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        with open(out_annotation_path, 'w') as f:
            for image_ppt in self.ppts_train:    
                #download image
                image_name = image_ppt[0][0]
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.train_path, 'images'))        
                f.writelines([image_name, ' ', ' '.join(str(','.join(str(ppt) for ppt in ppts[1:])) 
                                                                for ppts in image_ppt), '\n'])
        #write to txt file
        test_out_annotation_dir = self.valid_path(os.path.join(self.test_path, 'annotations'))
        out_annotation_path = os.path.join(test_out_annotation_dir, 'annotations.txt')
        with open(out_annotation_path, 'w') as f:
            for image_ppt in self.ppts_test:   
                #download image
                image_name = image_ppt[0][0]
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.test_path, 'images'))         
                f.writelines([image_name, ' ', ' '.join(str(','.join(str(ppt) for ppt in ppts[1:])) 
                                                                for ppts in image_ppt), '\n'])
        #write to txt file
        validation_out_annotation_dir = self.valid_path(os.path.join(self.validation_path, 'annotations'))
        out_annotation_path = os.path.join(validation_out_annotation_dir, 'annotations.txt')
        with open(out_annotation_path, 'w') as f:
            for image_ppt in self.ppts_valid:   
                #download image
                image_name = image_ppt[0][0]
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.validation_path, 'images'))          
                f.writelines([image_name, ' ', ' '.join(str(','.join(str(ppt) for ppt in ppts[1:])) 
                                                                for ppts in image_ppt), '\n'])
        print('All done!') 

    def get_xml_template(self):
        request_url = 'https://rectvision.s3.amazonaws.com/xmlTemplate.xml'
        response = requests.get(request_url)
        open('template.xml', 'wb').write(response.content)

    def rectjson_to_xml(self):
        #get template
        self.get_xml_template()
        print('Starting conversion...')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        tree = ET.parse('template.xml')
        root = tree.getroot()
        for image_ppt in self.ppts_train:
            #download image
            image_name = image_ppt[0][0]
            image_url = self.annotations[image_name]['image_url']
            self.download_image(image_url, os.path.join(self.train_path, 'images'))    
            img_path, img_width, img_height, img_depth = image_ppt[0][:4]         
            #write to xml file
            train_out_annotation_dir = self.valid_path(os.path.join(self.train_path, 'annotations'))
            out_annotation_path = os.path.join(train_out_annotation_dir, self.replace_extension(img_path, '.xml'))
            #modify xml template
            folder = root.find('folder')
            folder.text = train_out_annotation_dir

            fname = root.find('filename')
            fname.text = img_path

            src = root.find('source')
            database = src.find('database')
            database.text = self.database

            size = root.find('size')
            width = size.find('width')
            width.text = str(img_width)
            height = size.find('height')
            height.text = str(img_height)
            depth = size.find('depth')
            depth.text = str(img_depth)

            for ppt in image_ppt:
                #append new object
                obj = ET.SubElement(root, 'object')

                name = ET.SubElement(obj, 'name')
                name.text = str(ppt[4])

                pose = ET.SubElement(obj, 'pose')
                pose.text = 'Unspecified'

                bndbox = ET.SubElement(obj, 'bndbox')

                xmin = ET.SubElement(bndbox, 'xmin')
                xmin.text = str(int(ppt[6]))
                xmax = ET.SubElement(bndbox, 'xmax')
                xmax.text = str(int(ppt[7]))
                ymin = ET.SubElement(bndbox, 'ymin')
                ymin.text = str(int(ppt[8]))
                ymax = ET.SubElement(bndbox, 'ymax')
                ymax.text = str(int(ppt[9]))

            #save annotation to xml file
            tree.write(out_annotation_path)
        
        for image_ppt in self.ppts_test:   
            #download image
            image_name = image_ppt[0][0]
            image_url = self.annotations[image_name]['image_url']
            self.download_image(image_url, os.path.join(self.test_path, 'images'))
            img_path, img_width, img_height, img_depth = image_ppt[0][:4]         
            #write to xml file
            test_out_annotation_dir = self.valid_path(os.path.join(self.test_path, 'annotations'))
            out_annotation_path = os.path.join(test_out_annotation_dir, self.replace_extension(img_path, '.xml'))
            #modify xml template
            folder = root.find('folder')
            folder.text = test_out_annotation_dir

            fname = root.find('filename')
            fname.text = img_path

            src = root.find('source')
            database = src.find('database')
            database.text = self.database

            size = root.find('size')
            width = size.find('width')
            width.text = str(img_width)
            height = size.find('height')
            height.text = str(img_height)
            depth = size.find('depth')
            depth.text = str(img_depth)

            for ppt in image_ppt:
                #append new object
                obj = ET.SubElement(root, 'object')

                name = ET.SubElement(obj, 'name')
                name.text = str(ppt[4])

                pose = ET.SubElement(obj, 'pose')
                pose.text = 'Unspecified'

                bndbox = ET.SubElement(obj, 'bndbox')

                xmin = ET.SubElement(bndbox, 'xmin')
                xmin.text = str(int(ppt[6]))
                xmax = ET.SubElement(bndbox, 'xmax')
                xmax.text = str(int(ppt[7]))
                ymin = ET.SubElement(bndbox, 'ymin')
                ymin.text = str(int(ppt[8]))
                ymax = ET.SubElement(bndbox, 'ymax')
                ymax.text = str(int(ppt[9]))

            #save annotation to xml file
            tree.write(out_annotation_path)
        
        for image_ppt in self.ppts_valid:   
            #download image
            image_name = image_ppt[0][0]
            image_url = self.annotations[image_name]['image_url']
            self.download_image(image_url, os.path.join(self.validation_path, 'images'))
            img_path, img_width, img_height, img_depth = image_ppt[0][:4]         
            #write to xml file
            validation_out_annotation_dir = self.valid_path(os.path.join(self.validation_path, 'annotations'))
            out_annotation_path = os.path.join(validation_out_annotation_dir, self.replace_extension(img_path, '.xml'))
            #modify xml template
            folder = root.find('folder')
            folder.text = validation_out_annotation_dir

            fname = root.find('filename')
            fname.text = img_path

            src = root.find('source')
            database = src.find('database')
            database.text = self.database

            size = root.find('size')
            width = size.find('width')
            width.text = str(img_width)
            height = size.find('height')
            height.text = str(img_height)
            depth = size.find('depth')
            depth.text = str(img_depth)

            for ppt in image_ppt:
                #append new object
                obj = ET.SubElement(root, 'object')

                name = ET.SubElement(obj, 'name')
                name.text = str(ppt[4])

                pose = ET.SubElement(obj, 'pose')
                pose.text = 'Unspecified'

                bndbox = ET.SubElement(obj, 'bndbox')

                xmin = ET.SubElement(bndbox, 'xmin')
                xmin.text = str(int(ppt[6]))
                xmax = ET.SubElement(bndbox, 'xmax')
                xmax.text = str(int(ppt[7]))
                ymin = ET.SubElement(bndbox, 'ymin')
                ymin.text = str(int(ppt[8]))
                ymax = ET.SubElement(bndbox, 'ymax')
                ymax.text = str(int(ppt[9]))

            #save annotation to xml file
            tree.write(out_annotation_path)
        print('All done!')

    def rectjson_to_kerasRetinanetCsv(self):
        print('Starting conversion...')
        #write to csv file
        train_out_annotation_dir = self.valid_path(os.path.join(self.train_path, 'annotations'))
        test_out_annotation_dir = self.valid_path(os.path.join(self.test_path, 'annotations'))
        validation_out_annotation_dir = self.valid_path(os.path.join(self.validation_path, 'annotations'))
        out_annotation_path_train = os.path.join(train_out_annotation_dir, 'annotations.csv')
        out_annotation_path_test = os.path.join(test_out_annotation_dir, 'annotations.csv')
        out_annotation_path_valid = os.path.join(validation_out_annotation_dir, 'annotations.csv')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        with open(out_annotation_path_train, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_train:  
                #download image
                image_name = image_ppt[0][0]
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.train_path, 'images'))          
                writer.writerows(image_ppt)
        with open(out_annotation_path_test, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_test:
                #download image
                image_name = image_ppt[0][0]
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.test_path, 'images'))            
                writer.writerows(image_ppt)
        with open(out_annotation_path_valid, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_valid:  
                #download image
                image_name = image_ppt[0][0]
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.validation_path, 'images'))          
                writer.writerows(image_ppt)

        print('All done!')

    def rectjson_to_tfCsv(self):
        print('Starting conversion...')
        #write to csv file
        train_out_annotation_dir = self.valid_path(os.path.join(self.train_path, 'annotations'))
        test_out_annotation_dir = self.valid_path(os.path.join(self.test_path, 'annotations'))
        validation_out_annotation_dir = self.valid_path(os.path.join(self.validation_path, 'annotations'))
        out_annotation_path_train = os.path.join(train_out_annotation_dir, 'annotations.csv')
        out_annotation_path_test = os.path.join(test_out_annotation_dir, 'annotations.csv')
        out_annotation_path_valid = os.path.join(validation_out_annotation_dir, 'annotations.csv')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        with open(out_annotation_path_train, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_train:  
                #download image
                image_name = image_ppt[0][0]
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.train_path, 'images'))
                writer.writerows(image_ppt)
        with open(out_annotation_path_test, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_test:   
                #download image
                image_name = image_ppt[0][0]
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.test_path, 'images'))         
                writer.writerows(image_ppt)
        with open(out_annotation_path_valid, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_valid:    
                #download image
                image_name = image_ppt[0][0]
                image_url = self.annotations[image_name]['image_url']
                self.download_image(image_url, os.path.join(self.validation_path, 'images'))        
                writer.writerows(image_ppt)
        print('All done!')
        

    


import os
import argparse
import json
import shutil

from labelme import utils
from datetime import datetime
import numpy as np
import glob
import PIL.Image


class NpEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.integer):
            return float(obj)
        if isinstance(obj, np.floating):
            return float(obj)
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)

class LabelmeToCoco():
    def __init__(self, ann_dir, out_coco_dir, project_desc=""):
        """
        :param labelme_json: the list of all labelme json file paths
        :param save_json_path: the path to save new json
        """
        self.ann_dir = ann_dir
        self.labelme_json = glob.glob(os.path.join(self.ann_dir, "*.json"))
        self.out_coco_dir = out_coco_dir
        self.save_json_path = os.path.join(self.valid_path(self.out_coco_dir), 'annotations.json')
        self.project_desc = project_desc
        self.info = {}
        self.images = []
        self.categories = []
        self.annotations = []
        self.label = []
        self.annID = 1
        self.height = 0
        self.width = 0

        self.save_json()

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path
        
    def gen_info(self):
        self.info["description"] = self.project_desc
        date = datetime.now()
        self.info["version"] = "1.0"
        # self.info["year"] = date.year
        self.info["date_created"] = str(date)

    def data_transfer(self):
        for num, json_file in enumerate(self.labelme_json):
            print(json_file)
            with open(json_file, "r") as fp:
                data = json.load(fp)
                self.images.append(self.image(data, num))
                for shapes in data["shapes"]:
                    label = shapes["label"]
                    if label not in self.label:
                        self.label.append(label)
                    points = shapes["points"]
                    self.annotations.append(self.annotation(points, label, num))
                    self.annID += 1

        # Sort all text labels so they are in the same order across data splits.
        self.label.sort()
        for label in self.label:
            self.categories.append(self.category(label))
        for annotation in self.annotations:
            annotation["category_id"] = self.getcatid(annotation["category_id"])

    def image(self, data, num):
        '''This method gets the width, height, id and filename for an image and 
        stores it in the image dictionary'''
        image = {}
        img = utils.img_b64_to_arr(data["imageData"])
        height, width = img.shape[:2]
        img = None
        image["height"] = height
        image["width"] = width
        image["id"] = num
        image["file_name"] = data["imagePath"].split("/")[-1]

        self.height = height
        self.width = width

        return image

    def category(self, label):
        category = {}
        category["supercategory"] = label
        category["id"] = len(self.categories)
        category["name"] = label
        return category

    def annotation(self, points, label, num):
        annotation = {}
        contour = np.array(points)
        #x and y coordinates
        x = contour[:, 0]
        y = contour[:, 1]
        #area of bounding polgon
        # area = 0.5 * np.abs(np.dot(x, np.roll(y, 1)) - np.dot(y, np.roll(x, 1)))
        # annotation["segmentation"] = [[list(np.asarray(points).flatten())]]
        annotation["iscrowd"] = 0
        # annotation["area"] = area
        annotation["image_id"] = num
        [[xmin, ymin], [xmax, ymax]] = points
        o_width = xmax - xmin
        o_height = ymax - ymin
        annotation["bbox"] = [xmin, ymin, o_width, o_height]

        annotation["category_id"] = label  # self.getcatid(label)
        annotation["id"] = self.annID
        return annotation

    def getcatid(self, label):
        for category in self.categories:
            if label == category["name"]:
                return category["id"]
        print("label: {} not in categories: {}.".format(label, self.categories))
        exit()
        return -1

    def getbbox(self, points):
        polygons = points
        mask = self.polygons_to_mask([self.height, self.width], polygons)
        return self.mask2box(mask)

    def mask2box(self, mask):

        index = np.argwhere(mask == 1)
        rows = index[:, 0]
        clos = index[:, 1]

        left_top_r = np.min(rows)  # y
        left_top_c = np.min(clos)  # x

        right_bottom_r = np.max(rows)
        right_bottom_c = np.max(clos)

        return [
            left_top_c,
            left_top_r,
            right_bottom_c - left_top_c,
            right_bottom_r - left_top_r,
        ]

    def polygons_to_mask(self, img_shape, polygons):
        mask = np.zeros(img_shape, dtype=np.uint8)
        mask = PIL.Image.fromarray(mask)
        xy = list(map(tuple, polygons))
        PIL.ImageDraw.Draw(mask).polygon(xy=xy, outline=1, fill=1)
        mask = np.array(mask, dtype=bool)
        return mask

    def data2coco(self):
        data_coco = {}
        data_coco["info"] = self.info
        data_coco["images"] = self.images        
        data_coco["annotations"] = self.annotations
        data_coco["categories"] = self.categories
        return data_coco

    def copy_images(self):
        for path in os.listdir(self.ann_dir):
            if not path.endswith(".json"):
                # Copy image to out_coco_dir
                shutil.copy(os.path.join(self.ann_dir, path), self.out_coco_dir)

    def save_json(self):
        print("save coco json")
        self.gen_info()
        self.data_transfer()
        self.data_coco = self.data2coco()

        print(self.save_json_path)
        os.makedirs(
            os.path.dirname(os.path.abspath(self.save_json_path)), exist_ok=True
        )
        json.dump(self.data_coco, open(self.save_json_path, "w"), indent=4, cls=NpEncoder)
        self.copy_images()



import argparse
import xml.etree.ElementTree as ET
import ast
import os
import shutil
import glob
import numpy as np


class XmlToDarknetTxt():
    def __init__(self, label_to_id_file_path, ann_dir, out_txt_dir):
        self.label_to_id = self.read_dictionary(label_to_id_file_path)
        self.ann_dir = ann_dir
        self.out_txt_dir = out_txt_dir
        self.ppts = []
        self.current_img_path = None
        self.current_img_width = 0
        self.current_img_height = 0

        self.xml_to_txt()

    def read_dictionary(self, dict_file_path):
        with open(dict_file_path, "r") as data:
            dictionary = ast.literal_eval(data.read())  
        return dictionary     
    
    def extract_info_from_xml(self, ann_path):
        self.ppts = []
        annotation_root = ET.parse(ann_path).getroot()
        current_img_path = annotation_root.findtext('path')
        if current_img_path is None:
            current_img_path = annotation_root.findtext('filename')
        self.current_img_path = current_img_path
        #get width and height of image
        size = annotation_root.find('size')
        self.current_img_width = int(size.findtext('width'))
        self.current_img_height = int(size.findtext('height'))
        
        for obj in annotation_root.findall('object'):
            #get label
            label = obj.findtext('name')
            label_id =  self.label_to_id[label]
            bndbox = obj.find('bndbox')
            x1 = int(float(bndbox.findtext('xmin'))) - 1
            y1 = int(float(bndbox.findtext('ymin'))) - 1
            x2 = int(float(bndbox.findtext('xmax')))
            y2 = int(float(bndbox.findtext('ymax')))
            width = x2 - x1
            height = y2-y1
            x_center = x1 + (width/2)
            y_center = y1 + (height/2)
            self.ppts.append([label_id, 
                              np.abs(x_center)/self.current_img_width, 
                              np.abs(y_center)/self.current_img_height, 
                              np.abs(width)/self.current_img_width, 
                              np.abs(height)/self.current_img_height, '\n'])

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def xml_to_txt(self):
        print('Starting conversion...')
        self.out_txt_dir = self.valid_path(self.out_txt_dir)
        for ann_path in glob.glob(os.path.join(self.ann_dir, '*.xml')):
            self.extract_info_from_xml(ann_path)
            
            #copy image from ann_dir and save to out_txt_dir
            shutil.copy(os.path.join(self.ann_dir, self.current_img_path), self.out_txt_dir)
            #write to text file
            out_txt_path = os.path.splitext(os.path.join(self.out_txt_dir, self.current_img_path))[0] + '.txt'
            with open(out_txt_path, 'w') as f:
                #write each object per line
                for point in self.ppts:
                    f.write(' '.join(str(ppt) for ppt in point))
                
        print('All done!')
        


    

    





import argparse
import xml.etree.ElementTree as ET
import ast
import os
import glob
import csv
import shutil


class XmlToTfCsv():
    def __init__(self, ann_dir, out_csv_dir):
        self.ann_dir = ann_dir
        self.out_csv_dir = out_csv_dir
        self.ppts = []
        self.current_img_path = None
        self.current_img_width = 0
        self.current_img_height = 0

        self.xml_to_csv()
    
    def extract_info_from_xml(self, ann_path):
        self.ppts = []
        annotation_root = ET.parse(ann_path).getroot()
        current_img_path = annotation_root.findtext('path')
        if current_img_path is None:
            current_img_path = annotation_root.findtext('filename')
        self.current_img_path = current_img_path
        #get width and height of image
        size = annotation_root.find('size')
        self.current_img_width = int(size.findtext('width'))
        self.current_img_height = int(size.findtext('height'))
        
        for obj in annotation_root.findall('object'):
            #get label
            label = obj.findtext('name')
            bndbox = obj.find('bndbox')
            x_min = int(float(bndbox.findtext('xmin'))) - 1
            y_min = int(float(bndbox.findtext('ymin'))) - 1
            x_max = int(float(bndbox.findtext('xmax')))
            y_max = int(float(bndbox.findtext('ymax')))
            self.ppts.append([self.current_img_path, label, self.current_img_width, self.current_img_height, x_min, y_min, x_max, y_max])

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def xml_to_csv(self):
        print('Starting conversion...')
        self.out_csv_dir = self.valid_path(self.out_csv_dir)
        out_csv_path = os.path.join(self.out_csv_dir, 'annotations.csv')
        with open(out_csv_path, 'w') as f:
            writer = csv.writer(f)
            for ann_path in glob.glob(os.path.join(self.ann_dir, '*.xml')):
                self.extract_info_from_xml(ann_path)
                writer.writerows(self.ppts)        

                #copy image from ann_dir and save to out_csv_dir
                shutil.copy(os.path.join(self.ann_dir, self.current_img_path), self.out_csv_dir)       
        print('All done!')
        



    

    





import numpy as np
import json
import glob
import shutil
import os
import argparse
import ast
from labelme import utils


class LabelmeToDarknetTxt():   
    def __init__(self, label_to_id_file_path, ann_dir, out_txt_dir):
        self.label_to_id = self.read_dictionary(label_to_id_file_path)
        self.ann_dir = ann_dir
        self.out_txt_dir = out_txt_dir
        self.ppts = []
        self.current_img_path = None
        self.current_img_width = 0
        self.current_img_height = 0
        
        self.json_to_txt()

    def read_dictionary(self, dict_file_path):
        with open(dict_file_path, "r") as data:
            dictionary = ast.literal_eval(data.read())  
        return dictionary     
    
    def extract_info_from_json(self, ann_path):
        self.ppts = []
        with open(ann_path, 'r') as fp:
            data = json.load(fp)
            #get path to original image
            self.current_img_path = data['imagePath']
            #get  image dimension from imageData
            self.current_img_height, self.current_img_width = utils.img_b64_to_arr(data["imageData"]).shape[:2]
            #get annotation coordinates and labels
            for shapes in data['shapes']:
                points = shapes['points']
                #get label for each set of points
                label = shapes['label']
                #convert label to ID from label_to_id dictionary
                label_id = self.label_to_id[label]
                #get properties of the bounding box
                x_center, y_center, width, height = self.pointsTobbox(points)
                self.ppts.append([label_id, 
                                  np.abs(x_center)/self.current_img_width, 
                                  np.abs(y_center)/self.current_img_height, 
                                  np.abs(width)/self.current_img_width, 
                                  np.abs(height)/self.current_img_height, '\n'])

    def pointsTobbox(self, points):
        '''This method converts the points array to the properties of the object's bounding box'''
        x1,y1 = points[0][:]
        x2,y2 = points[1][:]        
        width = x2 - x1
        height = y2-y1
        x_center = x1 + (width/2)
        y_center = y1 + (height/2)
        return x_center, y_center, width, height

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def json_to_txt(self):
        print('Starting conversion...')
        #checkk validity of output text dir and create one if it doesn't exist
        self.out_txt_dir = self.valid_path(self.out_txt_dir)
        #iterate over all json files in self.ann_dir and extract required info from each
        for ann_path in glob.glob(os.path.join(self.ann_dir, '*.json')):
            self.extract_info_from_json(ann_path)

            #copy image from ann_dir and save to out_txt_dir
            shutil.copy(os.path.join(self.ann_dir, self.current_img_path), self.out_txt_dir)
            #write to text file
            out_txt_path = os.path.splitext(os.path.join(self.out_txt_dir, self.current_img_path))[0] + '.txt'
            with open(out_txt_path, 'w') as f:
                #write each object per line
                for point in self.ppts:
                    f.write(' '.join(str(ppt) for ppt in point))
                
        print('All done!')
        
  

    